/* Replace the missing recording image */
function imageErrorHandler(img, whReceived, audioOnly) {
  img.onerror = "";
  // If audio only, display audio only image
  if (audioOnly) {
    img.src = addpipeAdmin.imgAudioOnly;
    return true;
  }
  // If the webhook was received, display 404 (recording was deleted); else display nothing
  img.src = whReceived ? addpipeAdmin.img404 : "";
  return true;
}

/* Sync deleted recordings - ajax button */
jQuery(document).ready(function ($) {
  // Handle button click for deleting a recording
  $(".delete-recording").click(function () {
    var btn = $(this);
    var recordingId = btn.data("id"); // Get the recording ID from the button's data attribute

    if (
      confirm(
        "Are you sure you want to delete this recording from the local WP database?"
      )
    ) {
      // Disable the button and show a deleting message
      btn.prop("disabled", true).html("Deleting...");

      // Prepare the data to send to the server
      var data = {
        action: "delete_recording", // This is the action name that will be handled in the PHP function
        recording_id: recordingId, // Pass the recording ID
        _wpnonce: addpipeAdmin._wpnonceDelete, // Nonce for security
      };

      // Send the AJAX request to the server
      $.post(addpipeAdmin.ajaxurl, data, function (response) {
        if (response.success) {
          $(`#${recordingId}`).remove(); // Remove the recording element from the page
        } else {
          btn.prop("disabled", false).html("Delete");
          alert("Failed to delete recording: " + response.data.message);
        }
      });
    }
  });

  // Handle button click for downloading recording
  $(".addpipe-download-recording").on("click", function (event) {
    event.preventDefault();
    let fileUrl = $(this).data("file");
    $.post(
      addpipeAdmin.ajaxurl,
      {
        action: "wp_ajax_addpipe_download_file",
        file: fileUrl,
        _wpnonce: addpipeAdmin._wpnonceDownload, // Nonce for security
      },
      function (response) {
        if (response && response.success) {
          let blob = new Blob([response.data], {
            type: "application/octet-stream",
          });
          let link = document.createElement("a");
          link.href = window.URL.createObjectURL(blob);
          link.download = fileUrl.split("/").pop();
          document.body.appendChild(link);
          link.click();
          document.body.removeChild(link);
        } else {
          console.error(
            "Download failed:",
            response.data?.message || "Unknown error"
          );
        }
      }
    ).fail(function (_, textStatus, errorThrown) {
      console.error("Download failed:", textStatus, errorThrown);
    });
  });

  var btnSyncDeleted = $(".sync-deleted-recordings");
  btnSyncDeleted.click(function () {
    $(this).html("Synchronizing...");

    setTimeout(function () {
      data = {
        action: "addpipe_ajax_sync_deleted",
        _wpnonce: addpipeAdmin._wpnonceSync,
      };
      $.post(addpipeAdmin.ajaxurl, data, function (response) {
        btnSyncDeleted.html(response);
      });
    }, 650);
  });

  // Open the webhook setup section
  $("#expand-webhook-section").click(() => {
    $("#webhook-section").slideToggle("slow");
  });
});
