<?php

// No direct access
if (!defined('ABSPATH')) {
    die('Error #845173685');
}

class AddPipe
{

    /**
     * Account hash holder
     * @var string
     */
    public $accountHash = '';

    /**
     * Recording environment ID holder
     * @var string
     */
    public $envId = '';

    /**
     * Pipe webhook key
     * @var string
     */
    public $pipeWebhookKey = '';

    /**
     * Recording width holder
     * @var string
     */
    public $widthVal = '600px';

    /**
     * Recording height holder
     * @var int
     */
    public $heightVal = 390;

    /**
     * Recording length holder
     * @var int
     */
    public $recordingLength = 20;

    /**
     * Recording quality holder
     * @var string
     */
    public $recordingQuality = '480p';

    /**
     * Auto Save Recording holder
     * @var string
     */
    public $asv = '1';

    /**
     * Audio Only holder
     * @var string
     */
    public $ao = '0';

    /**
     * Show Menu holder
     * @var string
     */
    public $showMenu = '1';

    /**
     * Skip Initial Screen holder
     * @var string
     */
    public $sis = '0';

    /**
     * Desktop Upload holder
     * @var string
     */
    public $dup = '0';

    /**
     * Screen + Camera rec holder
     * @var string
     */
    public $srec = '0';

    /**
     * Countdown timer holder
     * @var string
     */
    public $countdowntimer = '0';

    /**
     * Background blurring holder
     * @var string
     */
    public $bgblur = '0';

    /**
     * Plugin instance
     * @var
     */
    private static $_instance;

    /**
     * Plugin screen name
     * @var string
     */
    public $_pluginScreenName = 'AddPipe';

    /**
     * L'Constructeur
     */
    public function __construct()
    {
        // Get the user's account hash from the database
        $this->accountHash = $this->addpipeGetAccountHash();

        // Retrieve the environment code from the database
        $this->envId = $this->addpipeGetEnvId();

        // Retrieve the webhook key from the database
        $this->pipeWebhookKey = $this->addpipeGetPipeWebhookKey();
    }

    /**
     * Hook our methods into WordPress
     */
    public function run()
    {
        /* Draw the menu */
        add_action('admin_menu', array($this, 'addpipeDrawMenu'));

        /* AJAX: save settings */
        add_action('wp_ajax_addpipe_ajax_save_settings', array($this, 'addpipe_ajax_save_settings'));
        /* AJAX: Generate a shortcode for the recording client */
        add_action('wp_ajax_addpipe_ajax_shortcode_generator', array($this, 'addpipe_ajax_shortcode_generator'));
        /* AJAX: Download recordings from list on click */
        add_action('wp_ajax_addpipe_download_file', 'addpipe_handle_download');
        /* Hook the function to handle the AJAX request */
        add_action('wp_ajax_delete_recording', array($this, 'addpipe_ajax_delete_recording'));
        /* AJAX: sync deleted recordings */
        add_action('wp_ajax_addpipe_ajax_sync_deleted', array($this, 'addpipe_ajax_sync_deleted'));

        /* Listen to incoming hooks from addpipe.com */
        add_action('wp_ajax_nopriv_addpipeWebhook', array($this, 'addpipeWebhook'));
        /* The [pipe_recorder] shortcode */
        add_shortcode('pipe_recorder', function ($atts) {
            return wp_kses($this->newRecorder($atts), $this->addpipe_allowed_html());
        });
        /* The [pipe_playback] */
        add_shortcode('pipe_playback', function ($atts) {
            return wp_kses($this->newPlayback($atts), $this->addpipe_allowed_html());
        });

        // Enqueue scripts (css and js) in the admin area - only on our plugin pages
        add_action('admin_enqueue_scripts', array($this, 'addpipeEnqueueBackendScripts'));
        // Enqueue scripts (css and js) in the frontend area
        add_action('wp_enqueue_scripts', array($this, 'addpipeEnqueueFrontendScripts'));
        // Enqueue scripts in the admin area selectively
        add_action('admin_enqueue_scripts', array($this, 'addpipeSelectivelyEnqueueAdminScripts'));
    }

    public function addpipeEnqueueBackendScripts()
    {
        // Check if we are inside the plugin's menu
        if (isset($_GET['page']) && strpos(sanitize_text_field(wp_unslash($_GET['page'] ?? '')), 'pipe-recorder') === 0) {
            wp_enqueue_script('pipejs', 'https://cdn.addpipe.com/2.0/pipe.min.js', array(), '2.0', false);
            wp_enqueue_style('pipecss', 'https://cdn.addpipe.com/2.0/pipe.css', array(), '2.0');
        }
    }

    /**
     * Hook to enqueue scripts (css and js) in the WordPress admin
     */
    public function addpipeSelectivelyEnqueueAdminScripts() {
        // Register and enqueue the external admin CSS file
        if (!isset($_GET['page'])) return;
        if (strpos(sanitize_text_field(wp_unslash($_GET['page'] ?? '')), 'embed-recorder')) {
            // Register and enqueue the external admin CSS file
            wp_register_style('addpipe-embedRecorder-style', plugin_dir_url(__DIR__) . 'css/admin/embedRecorder.css', array(), '2.0');
            wp_enqueue_style('addpipe-embedRecorder-style');

            // Enqueue the external admin JS file
            wp_register_script('addpipe-embedRecorder-script', plugin_dir_url(__DIR__) . 'js/admin/embedRecorder.js', array(), '2.0', false);
            wp_enqueue_script('addpipe-embedRecorder-script');
            // Pass PHP nonce variable to JavaScript using wp_localize_script
            wp_localize_script('addpipe-embedRecorder-script', 'addpipeAdmin', array(
                '_wpnonce' => esc_attr(wp_create_nonce('addpipe_ajax_shortcode_generator'))
            ));
            return;
        } else if (strpos(sanitize_text_field(wp_unslash($_GET['page'] ?? '')), 'recordings')) {
            // Register and enqueue the external admin CSS file
            wp_register_style('addpipe-recordedRecordings-style', plugin_dir_url(__DIR__) . 'css/admin/recordedRecordings.css', array(), '2.0');
            wp_enqueue_style('addpipe-recordedRecordings-style');

            // Enqueue the external admin JS file
            wp_register_script('addpipe-recordedRecordings-script', plugin_dir_url(__DIR__) . 'js/admin/recordedRecordings.js', array(), '2.0', false);
            wp_enqueue_script('addpipe-recordedRecordings-script');
            // Pass PHP nonce variable to JavaScript using wp_localize_script
            wp_localize_script('addpipe-recordedRecordings-script', 'addpipeAdmin', array(
                'imgAudioOnly' => esc_url($this->addpipeGetPluginViewURL('img/audio_only.png')),
                'img404' => esc_url($this->addpipeGetPluginViewURL('img/404.png')),
                '_wpnonceDelete' => esc_attr(wp_create_nonce('delete_recording_nonce')),
                '_wpnonceSync' => esc_attr(wp_create_nonce('addpipe_ajax_sync_deleted')),
                '_wpnonceDownload' => esc_attr(wp_create_nonce('addpipe_delete_recording')),
                'ajaxurl' => admin_url('admin-ajax.php')
            ));
            return;
        } else if (strpos(sanitize_text_field(wp_unslash($_GET['page'] ?? '')), 'settings')) {
            // Register and enqueue the external admin CSS file
            wp_register_style('addpipe-settings-style', plugin_dir_url(__DIR__) . 'css/admin/settings.css', array(), '2.0');
            wp_enqueue_style('addpipe-settings-style');

            // Enqueue the external admin JS file
            wp_register_script('addpipe-settings-script', plugin_dir_url(__DIR__) . 'js/admin/settings.js', array(), '2.0', false);
            wp_enqueue_script('addpipe-settings-script');
            // Pass PHP nonce variable to JavaScript using wp_localize_script
            wp_localize_script('addpipe-settings-script', 'addpipeAdmin', array(
                '_wpnonce' => esc_attr(wp_create_nonce('addpipe_ajax_save_settings'))
            ));
            return;
        } else if (strpos(sanitize_text_field(wp_unslash($_GET['page'] ?? '')), 'pipe-recorder') === 0) {
            // Register and enqueue the external admin CSS file
            wp_register_style('addpipe-recordNew-style', plugin_dir_url(__DIR__) . 'css/admin/recordNew.css', array(), '2.0');
            wp_enqueue_style('addpipe-recordNew-style');

            // Enqueue the external admin JS file
            wp_register_script('addpipe-recordNew-script', plugin_dir_url(__DIR__) . 'js/admin/recordNew.js', array(), '2.0', false);
            wp_enqueue_script('addpipe-recordNew-script');
            // Pass PHP nonce variable to JavaScript using wp_localize_script
            wp_localize_script('addpipe-recordNew-script', 'addpipeAdmin', array(
                'accountHash' => esc_attr($this->accountHash),
                'payload' => esc_attr($this->accountHash) . ',' . esc_attr(get_current_user_id()) . ',' . esc_attr($this->addpipeGetThePostId())
            ));
            return;
        }
    }

    public function addpipeEnqueueFrontendScripts()
    {
        if (is_singular()) {
            global $post;
    
            // Check if the content contains any of the shortcodes
            if (has_shortcode($post->post_content, 'pipe_recorder')) {
                wp_enqueue_script('pipejs', 'https://cdn.addpipe.com/2.0/pipe.min.js', array(), '2.0', false);
                wp_enqueue_style('pipecss', 'https://cdn.addpipe.com/2.0/pipe.css', array(), '2.0');
            }
        }
    }

    // Function to initialize WP_Filesystem
    public function addpipe_initialize_wp_filesystem()
    {
        if (!function_exists('WP_Filesystem')) {
            require_once(ABSPATH . 'wp-admin/includes/file.php');
        }

        WP_Filesystem();
    }


    /**
     * Install method
     */
    static function addpipeInstall()
    {
        // Include the necessary file for dbDelta function
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // General settings
        update_option('AddPipe' . 'AccountHash', '');
        update_option('AddPipe' . 'EnvId', '');
        update_option('AddPipe' . 'PipeWebhookKey', '');

        // Install the custom tables
        global $wpdb;

        // Define the table schema
        $charset_collate = $wpdb->get_charset_collate();

        $sql = "
                      CREATE TABLE IF NOT EXISTS
                      `{$wpdb->prefix}addpipe_records`
                        (
                        `internal_id` int unsigned NOT NULL AUTO_INCREMENT,
                        `user_id` bigint unsigned DEFAULT NULL,
                        `environment_code` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `stream_name` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `recording_id` int NOT NULL,
                        `recording_length` decimal(9,2) DEFAULT NULL,
                        `recording_size` int DEFAULT NULL,
                        `recording_url` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `recording_raw_url` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `recording_img_url` varchar(255) COLLATE utf8mb4_unicode_520_ci DEFAULT NULL,
                        `recording_width` SMALLINT(255) unsigned DEFAULT NULL,
                        `recording_height` SMALLINT(255) unsigned DEFAULT NULL,
                        `post_id` bigint unsigned DEFAULT '0',
                        `extension` VARCHAR(25),
                        `date` timestamp NULL DEFAULT CURRENT_TIMESTAMP,
                        `json` varchar(3000) CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_520_ci NOT NULL,
                        PRIMARY KEY (`internal_id`)
                        ) $charset_collate;
                        ";

        // Execute the query using dbDelta
        dbDelta($sql);

        // Remove "recording_"
        $sql = "
                      CREATE TABLE IF NOT EXISTS
                      `{$wpdb->prefix}addpipe_shortcodes`
                        (
                        `shortcode_id` INT(6) NOT NULL AUTO_INCREMENT,
                        `length` INT(11),
                        `quality` VARCHAR(5),
                        `env_code` CHAR(6),
                        `width` VARCHAR(25), 
                        `height` INT(20),
                        `asv` SMALLINT(1) DEFAULT 1,
                        `ao` SMALLINT(1) DEFAULT 0,
                        `showMenu` SMALLINT(1) DEFAULT 1,
                        `sis` SMALLINT(1) DEFAULT 0,
                        `dup` SMALLINT(1) DEFAULT 0,
                        `srec` SMALLINT(1) DEFAULT 0,
                        `countdowntimer` INT(11) DEFAULT 0,
                        `bgblur` SMALLINT(1) DEFAULT 0,
                        `post_id` INT(11) DEFAULT 0,
                        `date` TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                        PRIMARY KEY (`shortcode_id`)
                        ) $charset_collate;
        ";

        // Execute the query using dbDelta
        dbDelta($sql);


        //Add the pipe_access_plugin capability to administrator role
        $role = get_role('administrator');

        //add the umbrella capability
        $role->add_cap('pipe_access_plugin', true);

        //add individual capabilities for each fo the 4 menus of the plugin
        $role->add_cap('pipe_access_record', true);
        $role->add_cap('pipe_access_embed', true);
        $role->add_cap('pipe_access_recordings', true);
        $role->add_cap('pipe_access_setup', true);


        // Log any error to file
        // file_put_contents(dirname(__FILE__) . '/_plugin_error_activation.log', ob_get_contents(), FILE_APPEND);
        if (!function_exists('WP_Filesystem')) {
            require_once(ABSPATH . 'wp-admin/includes/file.php');
        }

        WP_Filesystem();
        global $wp_filesystem;
        // Log any error to file
        $log_file_path = dirname(__FILE__) . '/_plugin_error_activation.log';
        $log_content = ob_get_contents();

        if ($wp_filesystem) {
            $wp_filesystem->put_contents($log_file_path, $log_content, FILE_APPEND);
        }
    }

    /**
     * Uninstall plugin method
     */
    static function addpipeUninstall()
    {

        // Include the necessary file for dbDelta function
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        // Delete the general settings
        delete_option('AddPipe' . 'AccountHash');
        delete_option('AddPipe' . 'EnvId');
        delete_option('AddPipe' . 'PipeWebhookKey');

        //remove the capabilities added to the administrator
        $role = get_role('administrator');
        $role->remove_cap('pipe_access_plugin');
        $role->remove_cap('pipe_access_record');
        $role->remove_cap('pipe_access_embed');
        $role->remove_cap('pipe_access_recordings');
        $role->remove_cap('pipe_access_setup');

        // Delete custom tables
        global $wpdb;
        $sql = "DROP TABLE `{$wpdb->prefix}addpipe_records`";

        // Execute the query using dbDelta
        dbDelta($sql);

        $sql = "DROP TABLE `{$wpdb->prefix}addpipe_shortcodes`";

        // Execute the query using dbDelta
        dbDelta($sql);
    }

    /**
     * Returns the post/page ID, whether it's inside The Loop (or outside)
     * @return false|int
     */
    public function addpipeGetThePostId()
    {

        global $wpdb, $post, $wp_query;

        if (in_the_loop()) {
            return get_the_ID();
        } else {
            return $wp_query->get_queried_object_id();
        }

    }

    /**
     * Returns the AddPipe instance
     * @return AddPipe
     */
    static function addpipeGetInstance()
    {
        if (!isset(self::$_instance)) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    /**
     * Class name
     * @return string
     */
    public function __toString()
    {
        return 'AddPipe';
    }

    /**
     * Returns the plugin folder path
     * @return string
     */
    public function addpipeGetPluginFolder()
    {
        return dirname(dirname(__FILE__));
    }

    /**
     * Returns the plugin web address (URL)
     * @return string
     */
    public function addpipeGetPluginUrl()
    {
        return plugin_dir_url(__DIR__);
    }

    /**
     * Returns the plugin's server path, eg /srv/users/serverpilot/apps/dev/public/wordpress/wp-content/plugins/pipe-recorder/
     * @return string
     */
    public function addpipeGetPluginPath()
    {
        return plugin_dir_path(__FILE__);
    }

    /**
     * Return the URL to the static folder (css, js, img)
     * @param $staticFileOrFolder
     * @return string
     */
    public function addpipeGetPluginViewURL($staticFileOrFolder)
    {
        return $this->addpipeGetPluginUrl() . 'static/' . $staticFileOrFolder;
    }

    /**
     * Returns the path to the Views folder
     * @return string
     */
    public function addpipeGetViewsPath()
    {
        return $this->addpipeGetPluginPath() . 'views/';
    }

    /**
     * Loads the appropriate controller based on URL
     * @return AddPipeRecordedRecordings|AddPipeRecordNew|AddPipeSettings|AddPipeEmbedRecorder
     * @throws Exception
     */
    public function addpipeLoadController()
    {
        if (isset($_GET) && !empty($_GET['page'])) {
            $page = sanitize_text_field(wp_unslash($_GET['page'] ?? ''));
            switch ($page):
                case 'pipe-recorder/load/settings':
                    require_once $this->addpipeGetPluginPath() . 'Settings.php';
                    return new AddPipeSettings();
                    break;
                case 'pipe-recorder/load/recordings':
                    require_once $this->addpipeGetPluginPath() . 'RecordedRecordings.php';
                    return new AddPipeRecordedRecordings();
                    break;
                case 'pipe-recorder':
                    require_once $this->addpipeGetPluginPath() . 'RecordNew.php';
                    return new AddPipeRecordNew();
                    break;
                case 'pipe-recorder/load/embed-recorder':
                    require_once $this->addpipeGetPluginPath() . 'EmbedRecorder.php';
                    return new AddPipeEmbedRecorder();
                    break;
                default:
                    include_once $this->addpipeGetPluginPath() . '../views/not_found.php';
                    throw new Exception('Not found. #98645312');
            endswitch;
        }
    }

    /**
     * "Draws" the plugin menu
     * @return string
     */
    public function addpipeDrawMenu()
    {
        if (current_user_can('pipe_access_plugin')) {
            $this->_pluginScreenName = add_menu_page(
                // $page_title - The text to be displayed in the title tags of the page when the menu is selected
                'Pipe Recorder',
                // $menu_title - The on-screen name text for the menu
                'Pipe Recorder',
                // $capability - The capability required for this menu to be displayed to the user
                'pipe_access_plugin',
                // $menu_slug - The slug name to refer to this menu by (should be unique for this menu). Prior to Version 3.0 this was called the file (or handle) parameter.
                // If the function parameter is omitted, the menu_slug should be the PHP file that handles the display of the menu page content.
                'pipe-recorder',
                // function - (optional) The function that displays the page content for the menu page.
                // Default: None. Technically, the function parameter is optional, but if it is not supplied, then WordPress will assume that including the PHP file will generate the administration screen,
                // without calling a function. Most plugin authors choose to put the page-generating code in a function within their main plugin file. In the event that the function parameter is specified,
                // it is possible to use any string for the menu_slug parameter. This allows usage of pages such as ?page=my_super_plugin_page instead of ?page=my-super-plugin/admin-options.php.
                array($this, 'addpipeLoadController'),
                // $icon_url
                plugin_dir_url(__DIR__) . 'static/img/logo.png',
                // position
                81
            );

            // add_submenu_page($parent_slug, $page_title, $menu_title, $capability, $menu_slug, $function);
            add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Create Recording', 'pipe_access_record', 'pipe-recorder', array($this, 'addpipeLoadController'));
            add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Embed Recorder', 'pipe_access_embed', 'pipe-recorder/load/embed-recorder', array($this, 'addpipeLoadController'));
            add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Recordings', 'pipe_access_recordings', 'pipe-recorder/load/recordings', array($this, 'addpipeLoadController'));
            add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Settings', 'pipe_access_setup', 'pipe-recorder/load/settings', array($this, 'addpipeLoadController'));

            // If there is no account hash, probably the user doesn't have an addpipe.com account
//             if (empty($this->accountHash)) {
//                 add_submenu_page('pipe-recorder', 'Pipe Recorder', 'Get Trial', 'manage_options', 'pipe-recorder/load/get-trial', array($this, 'addpipeLoadController'));
//             }
        }

        // Log any error to file
        // file_put_contents(dirname(__FILE__) . '/_plugin_error_menu_draw.log', ob_get_contents(), FILE_APPEND);
        $this->addpipe_initialize_wp_filesystem();
        global $wp_filesystem;
        // Log any error to file
        $log_file_path = dirname(__FILE__) . '/_plugin_error_menu_draw.log';
        $log_content = ob_get_contents();

        if ($wp_filesystem) {
            $wp_filesystem->put_contents($log_file_path, $log_content, FILE_APPEND);
        }
    }

    /**
     * Listen to any webhooks from addpipe.com and validate requests using unique secret webhook key
     */
    public function addpipeWebhook()
    {
        // Check if the request has the correct content-type
        if (!isset($_SERVER['CONTENT_TYPE']) || $_SERVER['CONTENT_TYPE'] !== 'application/json') {
            wp_die('Unsupported content-type', 400);
        }

        // Dynamically get the webhook URL
        $webhook_url = admin_url('admin-ajax.php?action=addpipeWebhook');

        // Pipe webhook key
        $webhook_key = $this->pipeWebhookKey;

        if (!$webhook_key) {
            wp_die('Webhook key not set', 403);
        }

        // Retrieve the X-Pipe-Signature from headers
        $received_signature = $_SERVER['HTTP_X_PIPE_SIGNATURE'] ?? '';

        // Get the raw JSON payload
        $json_payload = file_get_contents('php://input');

        // Concatenate the webhook URL and the JSON payload
        $data_to_sign = $webhook_url . $json_payload;

        // Generate the HMAC-SHA1 hash and then base64-encode it
        $expected_signature = base64_encode(hash_hmac('sha1', $data_to_sign, $webhook_key, true));

        // Verify the signature
        if (!hash_equals($expected_signature, $received_signature)) {
            wp_die('Unauthorized request', 403);
        }

        $this->addpipe_initialize_wp_filesystem();
        global $wp_filesystem;
        global $wpdb;

        // Decode the raw POST data
        $json = json_decode($json_payload);

        if ($json && ($json->event == "video_recorded" || $json->event == "video_converted" || $json->event == "video_copied_pipe_s3")) {
            /* Slice the custom values from the payload */
            $payload = explode(',', $json->data->payload);
            $accountHash = filter_var($payload[0], FILTER_SANITIZE_STRING);
            $userId = filter_var($payload[1], FILTER_SANITIZE_NUMBER_INT);
            $postId = filter_var($payload[2], FILTER_SANITIZE_NUMBER_INT);
            // $envCode = filter_var($payload[3], FILTER_SANITIZE_STRING);

            /* Build the Insert query */
            $jsonData = json_encode($json); // Encode the Object back to json

            /* Check the contents and the account hash */
            if ($this->accountHash == $accountHash) {
                switch ($json->event) {
                    case "video_recorded":
                        $sql =
                            $wpdb->prepare(
                                "
                            INSERT INTO `{$wpdb->prefix}addpipe_records`
                            (
                                `user_id`,
                                `recording_id`,
                                `post_id`,
                                `environment_code`,
                                `stream_name`,
                                `extension`,
                                `json`
                            )
                            VALUES
                            (
                                %d, /* user_id from Payload, should be an integer */
                                %d, /* recording_id, should be an integer */
                                %d, /* post_id, should be an integer */
                                %s, /* environment_code from Payload, should be a string */
                                %s, /* stream_name, should be a string */
                                %s, /* extension, should be a string */
                                %s  /* jsonData, should be a string */
                            );
                            ",
                                array(
                                    $userId,
                                    $json->data->id,
                                    $postId,
                                    $json->data->envCode,
                                    $json->data->videoName,
                                    $json->data->type,
                                    $jsonData
                                )
                            )
                        ;

                        // Execute the query
                        $wpdb->query($sql);
                        break;
                    case "video_converted":
                        $sql =
                            $wpdb->prepare(
                                "
                            UPDATE `{$wpdb->prefix}addpipe_records` SET
                            `recording_size` = %d,
                            `recording_length` = %s,
                            `recording_width` = %d,
                            `recording_height` = %d
                            WHERE `recording_id` = %d;
                            ",
                                array(
                                    $json->data->size,
                                    $json->data->duration,
                                    $json->data->width,
                                    $json->data->height,
                                    $json->data->id
                                )
                            )
                        ;

                        // Execute the query
                        $wpdb->query($sql);
                        break;
                    case "video_copied_pipe_s3":
                        $sql =
                            $wpdb->prepare(
                                "
                            UPDATE `{$wpdb->prefix}addpipe_records` SET
                            `recording_url` = %s, 
                            `recording_img_url` = %s,
                            `recording_raw_url` = %s,
                            `recording_size` = %d
                            WHERE `recording_id` = %d;
                            ",
                                array(
                                    $json->data->url,
                                    $json->data->snapshotUrl,
                                    $json->data->rawRecordingUrl,
                                    $json->data->size,
                                    $json->data->id
                                )
                            )
                        ;

                        // Execute the query
                        $wpdb->query($sql);
                        break;
                    default:
                        // Unsupported type or empty JSON
                        http_response_code(400);
                        die(esc_attr("400 - Bad Request: Unsupported type or empty payload"));
                }

                // insert/update db ; give something back and die
                http_response_code(200);
                die('200 - OK.');
            } else {

                // The Payload isn't correct
                http_response_code(417);
                $_POST['payload'] .= sanitize_text_field(wp_unslash("------------- Error 417 -------------" . PHP_EOL));
                $log_file_path = dirname(__FILE__) . '/_addpipe_errors.log';
                $log_content = json_encode(sanitize_text_field(wp_unslash($_POST['payload'] ?? '')), true);

                if ($wp_filesystem) {
                    $wp_filesystem->put_contents($log_file_path, $log_content, PHP_EOL, LOCK_EX | FILE_APPEND);
                }
                die('417 - Expectation Failed. The Payload is not correct.');

            }

            // Default: $_POST['payload'] exists, but could not be proccessed
            http_response_code(501);
            $_POST['payload'] .= sanitize_text_field(wp_unslash("------------- Unknown error -------------" . PHP_EOL));
            $log_file_path = dirname(__FILE__) . '/_addpipe_errors.log';
            $log_content = json_encode(sanitize_text_field(wp_unslash($_POST['payload'] ?? '')), true);

            if ($wp_filesystem) {
                $wp_filesystem->put_contents($log_file_path, $log_content, PHP_EOL, LOCK_EX | FILE_APPEND);
            }
            die('501 - Not implemented. I cannot process the payload.');
        } else {
            http_response_code(400);
            die(esc_attr("400 - Bad Request: Unsupported event type: '{$json->event}'"));
        }
    }

    /**
     * Translates a given SIZE in a human readable format
     * @param $bytes
     * @param int $decimals
     * @return string
     */
    public function addpipeFilesizeForHumans($bytes, $decimals = 2)
    {
        $size = array(' B', ' KB', ' MB', ' GB', ' TB', ' PB', ' EB', ' ZB', ' YB');
        $factor = floor((strlen($bytes) - 1) / 3);

        return sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . @$size[$factor];
    }

    /**
     * Selects from DB only active recordings
     * @param int $limit
     * @param int $offset
     * @return array|null|object
     */
    public function addpipeGetRecordedRecordings($limit = 25, $offset = 0)
    {

        global $wpdb;

        $limit = filter_var($limit, FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($offset, FILTER_SANITIZE_NUMBER_INT);

        // Create a unique cache key based on the function arguments
        $cache_key = 'recorded_recordings_' . $limit . '_' . $offset;
        $cache_group = 'recorded_recordings_group';

        // Attempt to get the results from cache
        $results = wp_cache_get($cache_key, $cache_group);

        if ($results !== false) {
            return $results;
        }

        // return only 25 records, start on record 0 (OFFSET 0)
        // phpcs:disable
        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                                SELECT * FROM {$wpdb->prefix}addpipe_records
                                ORDER BY `date` DESC
                                LIMIT %d OFFSET %d
                                ",
                array(
                    $limit,
                    $offset
                )
            )
        );
        // phpcs:enable

        // Store the results in cache
        wp_cache_set($cache_key, $results, $cache_group, 30); // Cache for 1 minute

        return $results;
    }

    /**
     * Selects and returns from DB active recorder shortcodes
     * @param int $limit
     * @param int $offset
     * @return array|null|object
     */
    public function addpipeGetGeneratedShortcodes($limit = 25, $offset = 0)
    {

        global $wpdb;

        $limit = filter_var($limit, FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($offset, FILTER_SANITIZE_NUMBER_INT);

        // Create a unique cache key based on the function arguments
        $cache_key = 'addpipe_shortcodes_' . $limit . '_' . $offset;
        $cache_group = 'addpipe_shortcodes_group';

        // Attempt to get the results from cache
        $results = wp_cache_get($cache_key, $cache_group);

        if ($results !== false) {
            return $results;
        }

        // return only 25 records, start on record 0 (OFFSET 0)
        // phpcs:disable
        $results = $wpdb->get_results(
            $wpdb->prepare(
                "
                                SELECT * FROM {$wpdb->prefix}addpipe_shortcodes
                                ORDER BY `date` DESC
                                LIMIT %d OFFSET %d
                                ",
                array(
                    $limit,
                    $offset
                )
            )
        );
        // phpcs:enable

        // Store the results in cache
        wp_cache_set($cache_key, $results, $cache_group, 30); // Cache for 1 minute

        return $results;
    }

    /**
     * Check remote URL if exists (used to Sync Deleted Recordings)
     * @param $url
     * @return bool
     */
    public function addpipeIsFileOnServer($url)
    {
        if (empty($url))
            return true; // File has not been converted yet
        $serverResponse = wp_remote_get(filter_var($url, FILTER_SANITIZE_URL));
        $serverResponse = $serverResponse['response']['code'];

        if ($serverResponse === 403 || $serverResponse === 404) {
            return false;
        }

        return true;

    }

    /* ------------------------------------------------------------------------------------------
     * AJAX CALLS
     * ------------------------------------------------------------------------------------------
     */

    /**
     * AJAX: generate a shortcode for the recorder - EmbedRecorder.php
     */
    public function addpipe_ajax_shortcode_generator()
    {
        // Verify the nonce
        if (!isset($_POST['_wpnonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'addpipe_ajax_shortcode_generator')) {
            wp_send_json_error(['message' => 'Invalid nonce'], 403);
            return;
        }

        // Check user permissions
        if (!current_user_can('pipe_access_plugin')) {
            wp_send_json_error(['message' => 'Unauthorized access'], 403);
            return;
        }

        $length = isset($_POST['length']) ? filter_var(wp_unslash($_POST['length']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $quality = isset($_POST['quality']) ? sanitize_text_field(wp_unslash($_POST['quality'])) : '';
        $env = isset($_POST['env']) ? sanitize_text_field(wp_unslash($_POST['env'])) : '';
        $width = isset($_POST['width']) ? filter_var(wp_unslash($_POST['width']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $height = isset($_POST['height']) ? filter_var(wp_unslash($_POST['height']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $asv = isset($_POST['asv']) ? filter_var(wp_unslash($_POST['asv']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $ao = isset($_POST['ao']) ? filter_var(wp_unslash($_POST['ao']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $showMenu = isset($_POST['showMenu']) ? filter_var(wp_unslash($_POST['showMenu']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $sis = isset($_POST['sis']) ? filter_var(wp_unslash($_POST['sis']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $dup = isset($_POST['dup']) ? filter_var(wp_unslash($_POST['dup']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $srec = isset($_POST['srec']) ? filter_var(wp_unslash($_POST['srec']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $countdowntimer = isset($_POST['countdowntimer']) ? filter_var(wp_unslash($_POST['countdowntimer']), FILTER_SANITIZE_NUMBER_INT) : 0;
        $bgblur = isset($_POST['bgblur']) ? filter_var(wp_unslash($_POST['bgblur']), FILTER_SANITIZE_NUMBER_INT) : 0;

        // Define data to be inserted into DB
        $data = array(
            'length' => $length,
            'quality' => $quality,
            'env_code' => $env,
            'width' => $width,
            'height' => $height,
            'asv' => $asv,
            'ao' => $ao,
            'showMenu' => $showMenu,
            'sis' => $sis,
            'dup' => $dup,
            'srec' => $srec,
            'countdowntimer' => $countdowntimer,
            'bgblur' => $bgblur,
        );

        // Define data format
        $data_format = array('%d', '%s', '%s', '%s', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d');

        global $wpdb;
        // Perform the insert operation using prepared statement
        // phpcs:disable
        $result = $wpdb->insert("{$wpdb->prefix}addpipe_shortcodes", $data, $data_format);
        // phpcs:enable

        if ($result === false) {
            // Error occurred while inserting into the database
            $error_message = esc_attr($wpdb->last_error);
            echo wp_kses_post("<h3 style='color: red;'>Error occurred: $error_message</h3>");
        } else {
            // phpcs:disable
            $shortcode = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}addpipe_shortcodes` ORDER BY `date` DESC LIMIT 1");
            // phpcs:enable
            $shortcode = $shortcode[0];
            $id = esc_attr($shortcode->shortcode_id);

            echo wp_kses_post("
                <h3 style='color: darkgreen;'>Shortcode created </h3>
                <h4>Now you can use the shortcode <code>[pipe_recorder {$id}]</code> in any page or post.</h4>
            ");
        }

        // Kill the script, according to WP
        wp_die();
    }

    /**
     * AJAX: save data from the Settings.php
     */
    public function addpipe_ajax_save_settings()
    {
        if (
            !isset($_POST['_wpnonce']) ||
            !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'addpipe_ajax_save_settings')
        ) {
            wp_send_json_error(['message' => 'Invalid nonce'], 403);
        }
    
        // Ensure the user has the correct permissions
        if (!current_user_can('pipe_access_plugin')) {
            wp_send_json_error(['message' => 'Unauthorized access'], 403);
        }

        if (isset($_POST['accountHash'])) {
            $this->addpipeSetAccountHash(sanitize_text_field(wp_unslash($_POST['accountHash'] ?? '')));
        }

        if (isset($_POST['envId'])) {
            $this->addpipeSetEnvId(sanitize_text_field(wp_unslash($_POST['envId'] ?? '')));
        }

        if (isset($_POST['pipeWebhookKey'])) {
            $this->addpipeSetPipeWebhookKey(sanitize_text_field(wp_unslash($_POST['pipeWebhookKey'] ?? '')));
        }

        echo "
            <h4 style='color: green;'>Settings have been updated </h4>
            <p class='update-nag'>
                Your recordings will not be visible in the Recordings menu until you register the correct webhook. <a href='https://addpipe.com/docs/plugins/wordpress-plugin/' target='_blank'>Learn how</a>.
            </p>
        ";

        // Kill the script, according to WP
        wp_die();
    }

    /**
     * AJAX: delete the recording - RecordedRecordings.php
     */
    public function addpipe_ajax_delete_recording()
    {
        // Check if the request is valid
        if (isset($_POST['action']) && $_POST['action'] === 'delete_recording' && isset($_POST['_wpnonce']) && wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'delete_recording_nonce')) {
            global $wpdb;

            // Get the recording ID from the POST request
            $recording_id = intval(sanitize_text_field(wp_unslash($_POST['recording_id'] ?? '')));

            // Prepare the SQL query to delete the recording from the database
            $table_name = $wpdb->prefix . 'addpipe_records';
            $sql = $wpdb->prepare("DELETE FROM $table_name WHERE recording_id = %d", $recording_id);

            // Execute the query
            $deleted = $wpdb->query($sql);

            if ($deleted !== false) {
                // Send a success response
                wp_send_json_success(array('message' => 'Recording deleted successfully.'));
            } else {
                // Send a failure response
                wp_send_json_error(array('message' => 'Failed to delete recording.'));
            }

            wp_die(); // Terminate the script
        } else {
            // Send an error response if the request is not valid
            wp_send_json_error(array('message' => 'Invalid request.'));
            wp_die(); // Terminate the script
        }
    }

    public function addpipe_ajax_sync_deleted()
    {
        if (isset($_POST['action']) && $_POST['action'] === 'addpipe_ajax_sync_deleted' && isset($_POST['_wpnonce']) && wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'addpipe_ajax_sync_deleted')) {

            // Include the necessary file for dbDelta function
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

            global $wpdb;

            foreach ($this->addpipeGetRecordedRecordings() as $arr => $obj) {

                // Check the URL; if it returns 403 or 404, delete the entry from the database
                if ($this->addpipeIsFileOnServer($obj->recording_url)) {
                    echo esc_attr("Recording {$obj->recording_id} OK | ");
                } else {
                    echo esc_attr("Recording {$obj->recording_id} Deleted | ");
                    $sql = $wpdb->prepare("UPDATE {$wpdb->prefix}addpipe_records SET `active` = 0 WHERE `recording_url` = %s LIMIT 1", $obj->recording_url);
                    // Execute the query using dbDelta
                    dbDelta($sql);
                }

            }

            echo "<strong> Sync completed </strong>";

            // Kill the script, according to WP
            wp_die();

        } else {

            echo "Request failed. Error #846531";
            wp_die();
        }
    }

    /* ------------------------------------------------------------------------------------------
     * GETTERS AND SETTERS
     * ------------------------------------------------------------------------------------------
     */

    /**
     * Get the account hash
     * @return string
     */
    public function addpipeGetAccountHash()
    {
        $this->accountHash = get_option($this->_pluginScreenName . 'AccountHash');
        return (string) $this->accountHash;
    }

    /**
     * Get the environment code
     * @return string
     */
    public function addpipeGetEnvId()
    {
        $this->envId = get_option($this->_pluginScreenName . 'EnvId');
        return (string) $this->envId;
    }

    /**
     * Get the webhook key
     * @return string
     */
    public function addpipeGetPipeWebhookKey()
    {
        $this->pipeWebhookKey = get_option($this->_pluginScreenName . 'PipeWebhookKey');
        return (string) $this->pipeWebhookKey;
    }

    /**
     * Set the account hash
     * @param $accountHash
     * @return bool
     */
    public function addpipeSetAccountHash($accountHash)
    {
        $this->accountHash = filter_var(trim(rtrim($accountHash)), FILTER_SANITIZE_STRING);
        return update_option($this->_pluginScreenName . 'AccountHash', $accountHash);
    }

    /**
     * Set the environment code
     * @param $envId
     * @return bool
     */
    public function addpipeSetEnvId($envId)
    {
        $this->envId = filter_var(trim(rtrim($envId)), FILTER_SANITIZE_STRING);
        return update_option($this->_pluginScreenName . 'EnvId', $envId);
    }

    /**
     * Set the pipe webhook key
     * @param $pipeWebhookKey
     * @return bool
     */
    public function addpipeSetPipeWebhookKey($pipeWebhookKey)
    {
        $this->pipeWebhookKey = filter_var(trim(rtrim($pipeWebhookKey)), FILTER_SANITIZE_STRING);
        return update_option($this->_pluginScreenName . 'PipeWebhookKey', $pipeWebhookKey);
    }

    /**
     * New playback shortcode
     * @param $atts
     * @return string
     */
    public function newPlayback($atts)
    {

        global $wpdb;
        $recordingId = filter_var(wp_unslash($atts[0]), FILTER_SANITIZE_NUMBER_INT);
        $html = '';

        // Check if the user has placed a recording id
        if (empty($recordingId)) {
            return $html = "You must enter a recording id for the playback. Please copy the correct shortcode from the plugin's Recordings section.";
        }

        // Grab the recording
        // phpcs:disable
        $recordingDetails = $wpdb->get_results($wpdb->prepare("SELECT * FROM `{$wpdb->prefix}addpipe_records` WHERE `recording_id` = %d LIMIT 1", $recordingId));
        // phpcs:enable
        $recordingDetails = $recordingDetails[0];

        // Enqueue the CSS file and add inline styles
        wp_register_style('addpipe-newPlayback-style', plugin_dir_url(__DIR__) . 'css/newPlayback.css', array(), '2.0');
        wp_enqueue_style('addpipe-newPlayback-style');

        $html .= "
                <div class='addpipe-playback'>
                    <video class='player-playback' poster='" . esc_url($recordingDetails->recording_img_url) . "' controls>
                        <source src='" . esc_url($recordingDetails->recording_url) . "' type='video/mp4'>
                        <strong>Your browser does not support the video tag. Please use a newer browser (Chrome, Firefox).</strong>
                    </video>
                </div>
        ";

        return $html;

    }

    /**
     * Embed a new recorder - shortcode
     * * checks whether there are given values (as it should be)
     * * fault tolerant: uses the default values if no custom values are sent/set
     * @param $atts
     * @return string
     */
    public function newRecorder($atts)
    {
        global $wpdb;
        $recorderId = filter_var(wp_unslash($atts[0]), FILTER_SANITIZE_NUMBER_INT);
        // phpcs:disable
        $shortcodeRecord = $wpdb->get_results($wpdb->prepare("SELECT * FROM `{$wpdb->prefix}addpipe_shortcodes` WHERE `shortcode_id` = %d LIMIT 1", $recorderId));
        // phpcs:enable

        // Return empty string if no record is available
        if (count($shortcodeRecord) == 0) {
            return "";
        }

        $shortcodeRecord = $shortcodeRecord[0];

        // If the user forgot to enter a recorder id, we'll just use the default settings
        if (empty($recorderId)) {
            $shortcodeRecord = new stdClass();
            $shortcodeRecord->quality = $this->recordingQuality;
            $shortcodeRecord->length = $this->recordingLength;
        }

        $currentUserId = get_current_user_id();

        // Enqueue the embedded recorder script
        wp_register_script('addpipe-embedRecorder-client-script', plugin_dir_url(__DIR__) . 'js/embedRecorder.js', array(), '2.0', true);
        wp_enqueue_script('addpipe-embedRecorder-client-script');
        // Pass PHP variables to JavaScript using wp_localize_script
        wp_localize_script('addpipe-embedRecorder-client-script', 'addpipeClient', array(
            'id' => esc_attr('addpipe-recorder-' . $shortcodeRecord->shortcode_id),
            'width' => esc_attr($shortcodeRecord->width),
            'height' => esc_attr($shortcodeRecord->height),
            'qualityurl' => esc_attr('avq/' . $shortcodeRecord->quality . '.xml'),
            'accountHash' => esc_attr($this->accountHash),
            'eid' => esc_attr($shortcodeRecord->env_code),
            'mrt' => esc_attr($shortcodeRecord->length),
            'avrec' => esc_attr('1'),
            'asv' => esc_attr($shortcodeRecord->asv),
            'ao' => esc_attr($shortcodeRecord->ao),
            'showMenu' => esc_attr($shortcodeRecord->showMenu),
            'sis' => esc_attr($shortcodeRecord->sis),
            'dup' => esc_attr($shortcodeRecord->dup),
            'srec' => esc_attr($shortcodeRecord->srec),
            'countdowntimer' => esc_attr($shortcodeRecord->countdowntimer),
            'bgblur' => esc_attr($shortcodeRecord->bgblur),
            'payload' => esc_attr($this->accountHash . ',' . $currentUserId . ',' . $this->addpipeGetThePostId())
        ));

        // Enqueue the CSS file and add inline styles
        wp_register_style('addpipe-newRecorder-style', plugin_dir_url(__DIR__) . 'css/newRecorder.css', array(), '2.0');
        wp_enqueue_style('addpipe-newRecorder-style');

        // Add inline CSS to handle dynamic values
        $custom_css = "
            .addpipe-recorder {
                min-height: " . esc_attr($shortcodeRecord->height) . "px;
            }
        ";
        wp_add_inline_style('addpipe-newRecorder-style', $custom_css);

        $html = "<div id='addpipe-recorder-" . esc_attr($shortcodeRecord->shortcode_id) . "' class='addpipe-recorder'><p>Loading Pipe Recorder</p></div>";

        return $html;

    }


    /**
     * Define allowed HTML tags and attributes for safe output - used for outputting the recording client and recording playback
     */
    function addpipe_allowed_html()
    {
        return array(
            'div' => array('id' => array(), 'class' => array()),
            'video' => array('class' => array(), 'poster' => array(), 'controls' => array()),
            'source' => array('src' => array(), 'type' => array()),
            'strong' => array(),
            'p' => array(),
            'a' => array('href' => array(), 'title' => array(), 'class' => array())
        );
    }

    /**
     * Starts a download of the filed passed as $_POST['file']. Used for downloading recordings from list on click
     * @return never
     */
    public function addpipe_handle_download() {
        // Verify nonce
        if (!isset($_POST['_wpnonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'addpipe_delete_recording')) {
            wp_send_json_error(['message' => 'Invalid request'], 403);
        }
    
        // Validate and sanitize file URL
        $fileUrl = isset($_POST['file']) ? esc_url_raw(wp_unslash($_POST['file'])) : '';
    
        if (!$fileUrl || !filter_var($fileUrl, FILTER_VALIDATE_URL)) {
            wp_send_json_error(['message' => 'Invalid file URL'], 400);
        }
    
        // Only allow files from trusted domains
        $allowed_domains = ['addpipe.com'];
        $parsed_url = wp_parse_url($fileUrl);
        if (!isset($parsed_url['host']) || !in_array($parsed_url['host'], $allowed_domains, true)) {
            wp_send_json_error(['message' => 'Unauthorized domain'], 403);
        }
    
        // Fetch the file
        $fileContent = @file_get_contents($fileUrl);
        if ($fileContent === false) {
            wp_send_json_error(['message' => 'File not found'], 404);
        }
    
        // Send file as response
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename="' . basename($parsed_url['path']) . '"');
        header('Content-Length: ' . strlen($fileContent));
    
        echo esc_html($fileContent);
        exit;
    }

}
