<?php

// No direct access
if (!defined('ABSPATH')) {
    die('Error #845173685');
}

/*
 * Get all recordings
 *
 * $limit - how many to pull at once
 * $offset - from which index should we start displaying
 */
$recordings = $this->addpipeGetRecordedRecordings();

?>
<div class="wrap">

    <!-- Logo/title -->
    <h3>Manage your recordings</h3>

    <!-- If we have recordings, show the sync deleted option -->
    <!-- <?php echo (!empty($recordings)) ? "<h4>Have you deleted recordings from your addpipe.com account? Don't forget to <button class='sync-deleted-recordings'>Sync Deleted Recordings</button></h4>" : ''; ?> -->

    <!-- Start the listing -->
    <table class="widefat">
        <thead>
            <tr>
                <th>Date inserted</th>
                <th>Recording ID • Environment code</th>
                <th>Preview</th>
                <th>Recording details</th>
                <th>Actions</th>
                <th>Playback embed code</th>
                <th>Delete</th>
            </tr>
        </thead>
        <tbody>

            <!-- Check whether we have recordings -->
            <?php if (!empty($recordings)) { ?>
                <!-- Iterate through all recordings that are ACTIVE -->
                <?php foreach ($recordings as $arr => $obj) { ?>

                    <tr id='<?php echo esc_attr($obj->recording_id); ?>'>
                        <td>
                            <!-- Date -->
                            <?php echo esc_attr((new DateTime($obj->date, new DateTimeZone('UTC')))->setTimezone(wp_timezone())->format('Y-m-d H:i:s')); ?>
                        </td>
                        <td>
                            <!-- Recording ID -->
                            <?php echo esc_attr($obj->recording_id) . ' • ' . esc_attr($obj->environment_code); ?>
                        </td>
                        <td>
                            <!-- Img (snapshot/thumbnail) -->
                            <img src="<?php echo esc_url($obj->recording_img_url); ?>" width="150px" onerror="imageErrorHandler(this, <?php echo $obj->recording_img_url ? 'true' : 'false'; ?>, <?php echo ($obj->recording_img_url === 'audio_file') ? 'true' : 'false'; ?>);" />
                        </td>
                        <td>
                            <!-- Details (stream name, length, size) -->
                            Stream name: <code><?php echo esc_attr($obj->stream_name); ?></code>
                            <br />
                            Original extension: <code><?php echo esc_attr($obj->extension); ?></code>
                            <?php if ($obj->recording_size !== null && $obj->recording_size !== "0.00"): ?>
                                <br />
                                <?php echo esc_attr($this->addpipeFilesizeForHumans($obj->recording_size)); ?>
                            <?php endif; ?>
                            <?php if ($obj->recording_length !== null && $obj->recording_length !== "0.00"): ?>
                                <?php echo '• ' . esc_attr($obj->recording_length) . ' seconds'; ?>
                            <?php endif; ?>
                            <?php if ($obj->recording_width !== null && $obj->recording_width != 0 && $obj->recording_height !== null): ?>
                                <?php echo '• ' . esc_attr($obj->recording_width) . 'x' . esc_attr($obj->recording_height); ?>
                                <br />
                            <?php elseif ($obj->recording_width !== null && $obj->recording_width == 0): ?>
                                <?php echo '• audio only'; ?>
                                <br />
                            <?php else: ?>
                                <br />
                            <?php endif; ?>
                            By:
                            <!-- User who made the recording -->
                            <?php $user_data = get_userdata($obj->user_id); ?>
                            <?php echo ($user_data) ? esc_attr($user_data->user_login) : '<em>visitor</em>'; ?>
                        </td>
                        <td>
                            <!-- Control buttons: Play, Download MP4, Download RAW; -->
                            <?php if ($obj->recording_url !== null && $obj->recording_url !== ""): ?>
                                <a class="button-secondary playvideo slim-btn" href="<?php echo esc_url($obj->recording_url); ?>" target="_blank" title="Play MP4">Play MP4 </a>
                                <a class="button-secondary slim-btn" href="<?php echo esc_url($obj->recording_url); ?>" target="_blank" title="Download MP4">Download MP4</a>
                                <?php endif; ?>
                            <?php if ($obj->recording_raw_url !== null && $obj->recording_raw_url !== ""): ?>
                                <a class="button-secondary slim-btn" href="<?php echo esc_url($obj->recording_raw_url); ?>" target="_blank" title="Download RAW">Download <?php echo esc_html(strtoupper($obj->extension)); ?></a>
                            <?php endif; ?>
                        </td>
                        <td>
                            <p>
                                <!-- Shortcode input text -->
                                <?php if ($obj->recording_url !== null && $obj->recording_url !== ""): ?>
                                    <input type="text" readonly value="[pipe_playback <?php echo esc_attr($obj->recording_id); ?>]" />
                                <?php endif; ?>
                            </p>
                        </td>
                        <td>
                            <!-- Delete button -->
                            <button class="button-secondary delete-recording slim-btn" data-id="<?php echo esc_attr($obj->recording_id); ?>">Delete</button>
                        </td>
                    </tr>

                    <?php

                } // End foreach
            
            } else {

                echo '
                       <td colspan="6">
                        <h3>There are no entries in the local database.</h3>
                        <p>If you have recorded anything through the plugin but the recording does not show up here, make sure you`ve correctly set up the webhook in your <a href="https://addpipe.com" target="_blank">addpipe.com</a> account. <a href="https://addpipe.com/docs/plugins/wordpress-plugin/" target="_blank">Learn more</a>.</p>
                       </td>';
            }

            ?> <!-- End if check -->

        </tbody>
        <tfoot>
            <tr>
                <th>Date inserted</th>
                <th>Recording ID • Environment code</th>
                <th>Preview</th>
                <th>Recording details</th>
                <th>Action</th>
                <th>Playback embed code</th>
                <th>Delete</th>
            </tr>
        </tfoot>
    </table>
    <!-- Footer notes -->
    <p>The above table will only show recordings made through the <a href="<?php echo esc_url(admin_url()); ?>admin.php?page=pipe-recorder/load/embed-recorder">plugin's shortcodes</a> or through the plugin's <a href="<?php echo esc_url(admin_url()); ?>admin.php?page=pipe-recorder">create recording page</a>. Recordings made directly in the <a href="https://dashboard.addpipe.com/" target="_blank">Pipe account dashboard</a> will not show up here.</p>
    <p>Deleting a recording through the table above only deletes the entry from the local WP database. To delete the recording files and entry from the Pipe database you'll have to delete the recording from the <a href="https://dashboard.addpipe.com/recordings" target="_blank">Pipe account dashboard</a>.</p>
    <p>Snapshots, playback links and playback embed codes will work when you use Pipe Platform's complimentary storage and when the respective output is enabled in Pipe Platform's <a href="https://dashboard.addpipe.com/transcoding" target="_blank">Transcoding Engine</a>.</p>
    <p>Depending on the recording's video quality and it's length, it can take from a few seconds up to a couple of minutes for a recording to show up in the above table.</p>
    <p>For more information on the recordings above go to the Pipe account dashboard's <a href="https://dashboard.addpipe.com/recordings" target="_blank">Recordings section</a>.</p>
    <div id="dialog"></div>
</div>